/*
 * SPDX-FileCopyrightText: 2022-2023 Espressif Systems (Shanghai) CO LTD
 *
 * SPDX-License-Identifier: Apache-2.0
 */
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/unistd.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "hal/adc_types.h"
#include "soc/soc_caps.h"
#include "esp_log.h"
#include "esp_adc/adc_oneshot.h"
#include "esp_adc/adc_cali.h"
#include "esp_adc/adc_cali_scheme.h"

const static char *TAG = "EXAMPLE";

adc_channel_t channels[2] = {ADC_CHANNEL_0, ADC_CHANNEL_3};
adc_oneshot_unit_handle_t adc1_handle;
static int adc_raw[2];
static int voltage[2];


adc_cali_handle_t adc1_cali_chan0_handle = NULL;
adc_cali_handle_t adc1_cali_chan1_handle = NULL;
static bool adc_calibration_init(adc_unit_t unit, adc_channel_t channel, adc_atten_t atten, adc_cali_handle_t *out_handle);

void adc_init (adc_channel_t *channel, uint8_t numChannels)
{
	adc_oneshot_unit_init_cfg_t unitConfig = {
		.unit_id = ADC_UNIT_1,  // ADC1
	};
	adc_oneshot_new_unit(&unitConfig, &adc1_handle);
	
	adc_oneshot_chan_cfg_t channelConfig = {
		.atten = ADC_ATTEN_DB_12,  // 150mV - 2450mV
		.bitwidth = ADC_BITWIDTH_12,  // resolution 12 bit
	};
	
	for (int i=0; i<numChannels; i++)
	{
		adc_oneshot_config_channel(adc1_handle, channel[i], &channelConfig);
	}
	
    //-------------ADC1 Calibration Init---------------//

    adc_calibration_init(ADC_UNIT_1, channels[0], ADC_ATTEN_DB_12, &adc1_cali_chan0_handle);
    adc_calibration_init(ADC_UNIT_1, channels[1], ADC_ATTEN_DB_12, &adc1_cali_chan1_handle);
}


void app_main(void)
{
	adc_init(channels, 2);

    while (1) {
/*		for (int i=0; i<2; i++){
			adc_oneshot_read(adc1_handle, channels[i], &adc_raw[i]);
			voltage[i] = adc_raw[i]*3300/4095;
			printf ("CHANNEL%d\t RAW=%d\t mV=%d\n", channels[i], adc_raw[i], voltage[i]);
		}*/
		
		adc_oneshot_read(adc1_handle, channels[0], &adc_raw[0]);  // read channel 0
		adc_cali_raw_to_voltage(adc1_cali_chan0_handle, adc_raw[0], &voltage[0]);
		
		adc_oneshot_read(adc1_handle, channels[1], &adc_raw[1]);  // read channel 0
		adc_cali_raw_to_voltage(adc1_cali_chan1_handle, adc_raw[1], &voltage[1]);
		
		for (int i=0; i<2; i++){
		printf ("CHANNEL%d\t RAW=%d\t mV=%d\n", channels[i], adc_raw[i], voltage[i]);
		}
				
		usleep(500000);  // 500ms sleep
		
		}
}

/*---------------------------------------------------------------
        ADC Calibration
---------------------------------------------------------------*/
static bool adc_calibration_init(adc_unit_t unit, adc_channel_t channel, adc_atten_t atten, adc_cali_handle_t *out_handle)
{
    adc_cali_handle_t handle = NULL;
    esp_err_t ret = ESP_FAIL;
    bool calibrated = false;

#if ADC_CALI_SCHEME_CURVE_FITTING_SUPPORTED
    if (!calibrated) {
        ESP_LOGI(TAG, "calibration scheme version is %s", "Curve Fitting");
        adc_cali_curve_fitting_config_t cali_config = {
            .unit_id = unit,
            .chan = channel,
            .atten = atten,
            .bitwidth = ADC_BITWIDTH_DEFAULT,
        };
        ret = adc_cali_create_scheme_curve_fitting(&cali_config, &handle);
        if (ret == ESP_OK) {
            calibrated = true;
        }
    }
#endif

#if ADC_CALI_SCHEME_LINE_FITTING_SUPPORTED
    if (!calibrated) {
        ESP_LOGI(TAG, "calibration scheme version is %s", "Line Fitting");
        adc_cali_line_fitting_config_t cali_config = {
            .unit_id = unit,
            .atten = atten,
            .bitwidth = ADC_BITWIDTH_DEFAULT,
        };
        ret = adc_cali_create_scheme_line_fitting(&cali_config, &handle);
        if (ret == ESP_OK) {
            calibrated = true;
        }
    }
#endif

    *out_handle = handle;
    if (ret == ESP_OK) {
        ESP_LOGI(TAG, "Calibration Success");
    } else if (ret == ESP_ERR_NOT_SUPPORTED || !calibrated) {
        ESP_LOGW(TAG, "eFuse not burnt, skip software calibration");
    } else {
        ESP_LOGE(TAG, "Invalid arg or no memory");
    }

    return calibrated;
}
